/*
 * This file is part of firk's Common C Library
 *  string.c - openbsd-compatible wrapper for string.h/strings.h/w.e.
 *
 * (c) firk 2019, 2023; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#include "config.h"
#ifndef HAVE_STRCHRNUL
#ifdef HAVE_GNU_STRCHRNUL
#define _GNU_SOURCE
#endif
#endif
#include "types.h"
#include "string.h"

#ifdef FCL_EXPLICIT_BZERO
extern void explicit_bzero(void * p, fcl_size len) {
  bzero(p, len);
  __asm volatile ("" ::: "memory");
}
#endif

#ifdef FCL_STRLCPY
extern fcl_size strlcpy(char * dest, char const * src, fcl_size dstsize) {
  fcl_size pos;
  char c;
  pos = 0;
  if(dstsize) {
    dstsize--;
    while(pos<dstsize && (c=src[pos])) dest[pos++] = c;
    dest[pos] = 0;
  }
  if(src[pos]) pos += strlen(src+pos);
  return pos;
}
#endif

#ifndef HAVE_STRCHRNUL
extern char * fcl_strchrnul(const char * st, int c) {
#ifdef HAVE_GNU_STRCHRNUL
  return strchrnul(st, c);
#else
  while(*st && *st!=(char)c) st++;
  return (char*)st;
#endif
}
#endif

extern char * strrchrnul(const char * st, int c) {
  char const *p;
  for( ; *st; st++) if(*st==(char)c) {
    for(p=st++; *st; st++) if(*st==(char)c) p = st;
    return (char*)p;
  }
  return (char*)st;
}

extern fcl_size strpos(char const * st, int c) {
  return strchrnul(st,c) - st;
}

extern fcl_size strrpos(char const * st, int c) {
  return strrchrnul(st,c) - st;
}

extern fcl_size strnpos(char const * st, int c, fcl_size max) {
  size_t p;
  for(p=max; p && *st && *st!=(char)c; p--,st++);
  return max-p;
}


