/*
 * This file is part of firk's Common C Library
 *  strparse.c - string parsing functions
 *
 * (c) firk 2015, 2016, 2019, 2020, 2021, 2023; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#include <string.h>
#include "types.h"
#include "strparse.h"

extern char const * str_skip_spaces(char const * st) {
  char c;
  if(st) {
    while(c=*st) {
      if(c==' ' || c=='\t') st++;
      else break;
    }
  }
  return st;
}

extern char const * str_skip_prefix(char const * st, char const * prefix) {
  char c;
  if(!st) return NULL;
  while(c=*(prefix++)) if(c!=*(st++)) return NULL;
  return st;
}

extern char const * str_skip_untilsep(char const * st, char const * prefix,
                                      char const * sep) {
  char c;
  if(!st) return NULL;
  while(c=*(prefix++)) if(c!=*(st++)) return NULL;
  if(*st && (!sep || !strchr(sep, *st))) return NULL;
  return st;
}

extern char str_to_args(char * st, char * * argv, size_t * argc, size_t max,
                        uint flags) {
  char c, q;
  char *arg, *out;
  size_t n;
  n = 0;
  out = st;
  q = 0;
  while(1) {
    while((c=*st)==' ' || c=='\t') st++;
    if(!c) break;
    arg = out;
    while(1) {
      c = *st;
      if(!c || !q && (c==' ' || c=='\t')) break;
      if(c==q) { st++; q=0; continue; }
      if(!q && (c=='"' || c=='\'')) { st++; q=c; continue; }
      if(c=='\\') {
        c = st[1];
        if(!c) { *argc = n; return q?q:'\\'; }
        if(flags & 1) switch(c) {
        case 'a': c=7; break;
        case 'b': c=8; break;
        case 't': c=9; break;
        case 'n': c=10; break;
        case 'v': c=11; break;
        case 'f': c=12; break;
        case 'r': c=13; break;
        case '"': case '\'': break;
        default: *argc = n; return '\\';
        }
        st++;
      }
      *(out++) = c;
      st++;
    }
    if(q) { *argc = n; return q; }
    *(out++) = 0;
    if(n<max) argv[n] = arg;
    n++;
    if(!c) break;
    st++;
  }
  *argc = n;
  return 0;
}

extern unsigned char chr_to_value(char c) {
  if(c>='0' && c<='9') return (unsigned char)(c-'0');
  if(c>='A' && c<='Z') return (unsigned char)(c-'A'+10);
  if(c>='a' && c<='z') return (unsigned char)(c-'a'+10);
  return 255;
}

extern int str_get_bool(char const * st) {
  if(!st) return -1;
  if(!strcasecmp(st, "0") ||
     !strcasecmp(st, "no") ||
     !strcasecmp(st, "false") ||
     !strcasecmp(st, "n") ||
     !strcasecmp(st, "off")) return 0;
  if(!strcasecmp(st, "1") ||
     !strcasecmp(st, "yes") ||
     !strcasecmp(st, "true") ||
     !strcasecmp(st, "y") ||
     !strcasecmp(st, "on")) return 1;
  return -1;
}


