/*
 * This file is part of firk's Common C Library
 *  strparse_xs.c - string parsing functions for "xsize" typename
 *
 * (c) firk 2023; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#include <string.h>
#include "types.h"
#include "strparse.h"

#define APPLY_SUFFIX \
  switch(*s) {\
  case 'p':case 'P': rr = r*1024; if(r && rr/r!=1024) return -1; r = rr;\
  case 't':case 'T': rr = r*1024; if(r && rr/r!=1024) return -1; r = rr;\
  case 'g':case 'G': rr = r*1024; if(r && rr/r!=1024) return -1; r = rr;\
  case 'm':case 'M': rr = r*1024; if(r && rr/r!=1024) return -1; r = rr;\
  case 'k':case 'K': rr = r*1024; if(r && rr/r!=1024) return -1; r = rr; s++; \
  }

#define STR_TO_XI(btname, ftname, uname, sname) \
extern int str_to_u##ftname(char const * s, uname * res) {\
  uname r, rr;\
  if(!s || !*s) return -1;\
  if(strp_to_u##btname(&s, &r)<0) return -1;\
  APPLY_SUFFIX \
  if(*s) return -1;\
  *res = r;\
  return 0;\
} \
extern int strp_to_u##ftname(char const * * st, uname * res) {\
  char const * s;\
  uname r, rr;\
  if(!st || !(s=*st)) return -1;\
  if(strp_to_u##btname(&s, &r)<0) return -1;\
  APPLY_SUFFIX \
  *st = s;\
  *res = r;\
  return 0;\
}\
extern int str_hex_to_u##ftname(char const * s, uname * res) {\
  uname r, rr;\
  if(!s || !*s) return -1;\
  if(strp_base_to_u##btname(&s, &r, 16)<0) return -1;\
  APPLY_SUFFIX \
  if(*s) return -1;\
  *res = r;\
  return 0;\
}\
extern int str_base_to_u##ftname(char const * s, uname * res, uchar b) {\
  uname r, rr;\
  if(!s || !*s) return -1;\
  if(strp_base_to_u##btname(&s, &r, b)<0) return -1;\
  APPLY_SUFFIX \
  if(*s) return -1;\
  *res = r;\
  return 0;\
}\
extern int strp_base_to_u##ftname(char const * * st, uname * res, uchar b) {\
  char const * s;\
  uname r, rr;\
  if(!st || !(s=*st)) return -1;\
  if(strp_base_to_u##btname(&s, &r, b)<0) return -1;\
  APPLY_SUFFIX \
  *st = s;\
  *res = r;\
  return 0;\
}\
extern int str_to_s##ftname(char const * s, sname * res) {\
  sname r, rr;\
  if(!s || !*s) return -1;\
  if(strp_to_s##btname(&s, &r)<0) return -1;\
  APPLY_SUFFIX \
  if(*s) return -1;\
  *res = r;\
  return 0;\
}\
extern int strp_to_s##ftname(char const * * st, sname * res) {\
  char const * s;\
  sname r, rr;\
  if(!st || !(s=*st)) return -1;\
  if(strp_to_s##btname(&s, &r)<0) return -1;\
  APPLY_SUFFIX \
  *st = s;\
  *res = r;\
  return 0;\
}\
extern int str_base_to_s##ftname(char const * s, sname * res, uchar b) {\
  sname r, rr;\
  if(!s || !*s) return -1;\
  if(strp_base_to_s##btname(&s, &r, b)<0) return -1;\
  APPLY_SUFFIX \
  if(*s) return -1;\
  *res = r;\
  return 0;\
}\
extern int strp_base_to_s##ftname(char const * * st, sname * res, uchar b) {\
  char const * s;\
  sname r, rr;\
  if(!st || !(s=*st)) return -1;\
  if(strp_base_to_s##btname(&s, &r, b)<0) return -1;\
  APPLY_SUFFIX \
  *st = s;\
  *res = r;\
  return 0;\
}

STR_TO_XI(size, xsize, fcl_size, fcl_ssize)


